//------------------------ IMPORTS

import {Meteor} from "meteor/meteor";
import {Template} from "meteor/templating";
import {Session} from "meteor/session";
import {Cardsets} from "../../api/cardsets.js";
import {Cards} from "../../api/cards.js";
import {Ratings} from "../../api/ratings.js";
import {Paid} from "../../api/paid.js";
import {Leitner, Wozniak} from "../../api/learned.js";
import {Route} from "../../api/route.js";
import {ReactiveVar} from "meteor/reactive-var";
import "../card/card.js";
import "../learn/learn.js";
import "../presentation/presentation.js";
import "../forms/bonusForm.js";
import "../forms/cardsetForm.js";
import "./cardset.html";
import {CardType} from "../../api/cardTypes";
import {CardNavigation} from "../../api/cardNavigation";
import {CardVisuals} from "../../api/cardVisuals";
import {CardsetVisuals} from "../../api/cardsetVisuals";
import ResizeSensor from "../../../client/resize_sensor/ResizeSensor";
import {BertAlertVisuals} from "../../api/bertAlertVisuals";
import {Bonus} from "../../api/bonus";

Meteor.subscribe("cardsets");
Meteor.subscribe("paid");
Meteor.subscribe("notifications");

/**
 * Creates a web push subscription for the current device.
 * The Browser ask the user for permissions and creates the subscription.
 * Afterwards the subscription will be saved for the current user via the
 * Meteor-method addWebPushSubscription.
 */
function subscribeForPushNotification() {
	try {
		navigator.serviceWorker.getRegistration()
			.then(function (registration) {
				return registration.pushManager.getSubscription()
					.then(function (subscription) {
						if (!subscription) {
							return registration.pushManager.subscribe({userVisibleOnly: true});
						}
					});
			})
			.then(function (subscription) {
				if (subscription) {
					var rawKey = subscription.getKey ? subscription.getKey('p256dh') : '';
					const key = rawKey ? btoa(String.fromCharCode.apply(null, new Uint8Array(rawKey))) : '';
					var rawAuthSecret = subscription.getKey ? subscription.getKey('auth') : '';
					const authSecret = rawAuthSecret ? btoa(String.fromCharCode.apply(null, new Uint8Array(rawAuthSecret))) : '';
					const endpoint = subscription.endpoint;
					const sub = {
						endpoint: endpoint,
						key: key,
						authSecret: authSecret
					};
					Meteor.call("addWebPushSubscription", sub, function (error) {
						if (error) {
							throw new Meteor.Error(error.statusCode, 'Error subscription failed');
						}
					});
				}
			});
	} catch (error) {
		console.log(error);
	}
}

/**
 * Add the current user to the leitner algorithm.
 */
function addToLeitner(cardset_id) {
	subscribeForPushNotification();
	Meteor.call('addToLeitner', cardset_id);
}

function changeCollapseIcon(iconId) {
	if ($(iconId).hasClass("glyphicon-collapse-down")) {
		$(iconId).removeClass("glyphicon-collapse-down");
		$(iconId).addClass("glyphicon-collapse-up");
	} else {
		$(iconId).removeClass("glyphicon-collapse-up");
		$(iconId).addClass("glyphicon-collapse-down");
	}
}

/*
 * ############################################################################
 * cardset
 * ############################################################################
 */

Template.cardset.onCreated(function () {
	Session.set('ratingsLoaded', false);
	Meteor.subscribe('ratings', function () {
		Session.set('ratingsLoaded', true);
	});
	if (Session.get('activeCardset') === undefined || Session.get('activeCardset')._id !== Router.current().params._id) {
		Session.set('activeCardset', Cardsets.findOne(Router.current().params._id));
		Session.set('activeCard', undefined);
	}
	if (Number(Session.get('activeCard')) === Number(-1)) {
		Session.set('activeCard', undefined);
	}
	Session.set('shuffled', Cardsets.findOne(Router.current().params._id).shuffled);
	Session.set('cameFromEditMode', false);
	CardNavigation.toggleVisibility(true);
});

Template.cardset.rendered = function () {
	var customerId = Meteor.user().customerId;
	if ($('#payment-form').length) {
		Meteor.call('getClientToken', customerId, function (error, clientToken) {
			if (error) {
				throw new Meteor.Error(error.statusCode, 'Error getting client token from braintree');
			} else {
				braintree.setup(clientToken, "dropin", {
					container: "payment-form",
					onPaymentMethodReceived: function (response) {
						$('#buyCardsetBtn').prop("disabled", true);

						var nonce = response.nonce;

						Meteor.call('btCreateTransaction', nonce, Router.current().params._id, function (error) {
							if (error) {
								throw new Meteor.Error('transaction-creation-failed');
							} else {
								BertAlertVisuals.displayBertAlert(TAPi18n.__('cardset.money.bought'), 'success', 'growl-top-left');
							}
						});
					}
				});
			}
		});
	}
	$('html, body').animate({scrollTop: '0px'}, 0);
};

Template.cardset.helpers({
	'selectedForLearning': function () {
		if (Session.get('selectingCardsetToLearn')) {
			addToLeitner(this._id);
			Meteor.call("addWozniakCards", this._id);
			Session.set("selectingCardsetToLearn", false);
			BertAlertVisuals.displayBertAlert(TAPi18n.__('cardset.alert.addedToWorkload'), 'success', 'growl-top-left');
		}
	},
	isInBonus: function () {
		return Bonus.isInBonus(Session.get('activeCardset')._id, Meteor.userId());
	}
});

Template.cardset.events({
	'click #cardSetDelete': function () {
		$("#cardSetDelete").css('display', "none");
		$("#cardSetConfirm").css('display', "");

		$('#setCardsetFormModal').on('hidden.bs.modal', function () {
			$("#cardSetDelete").css('display', "");
			$("#cardSetConfirm").css('display', "none");
		});
	},
	'click #cardSetConfirm': function () {
		var id = this._id;

		$('#setCardsetFormModal').on('hidden.bs.modal', function () {
			Meteor.call("deleteCardset", id);
			Router.go('create');
		}).modal('hide');
	},
	'click #acceptRequest': function () {
		Meteor.call("acceptProRequest", this._id);
		BertAlertVisuals.displayBertAlert(TAPi18n.__('cardset.request.accepted'), 'success', 'growl-top-left');
		Router.go('home');
	},
	'click #declineRequest': function () {
		var reason = $('#declineRequestReason').val();
		if (reason === '') {
			BertAlertVisuals.displayBertAlert(TAPi18n.__('cardset.request.reason'), 'danger', 'growl-top-left');
		} else {
			Meteor.call("addNotification", this.owner, "Freischaltung des Kartensatzes " + this.name + " nicht stattgegeben", reason, this._id, TAPi18n.__('set-list.author'));
			Meteor.call("declineProRequest", this._id);
			BertAlertVisuals.displayBertAlert(TAPi18n.__('cardset.request.declined'), 'info', 'growl-top-left');
			Router.go('home');
		}
	},
	'click #backToCardsetDetailView, cllick #backToCardsetDetailViewFullscreen': function () {
		Router.go('cardsetdetailsid', {
			_id: Router.current().params._id
		});
	},
	'click .projectorIcon': function () {
		Router.go('presentation', {
			_id: Router.current().params._id
		});
	}
});

/*
 * ############################################################################
 * cardsetPreview
 * ############################################################################
 */

Template.cardsetPreview.events({
	"click #buyProBtn": function () {
		Router.go('profileMembership', {
			_id: Meteor.userId()
		});
	},
	"click #showPreviewHelp": function () {
		event.stopPropagation();
		Session.set('helpFilter', "previewCardset");
		Router.go('help');
	}
});

Template.cardsetPreview.onCreated(function () {
	if (Router.current().params._id) {
		let cardset = Cardsets.findOne({_id: Router.current().params._id}, {fields: {_id: 1, cardGroups: 1}});
		if (cardset !== undefined) {
			let filterQuery = {
				$or: [
					{cardset_id: cardset._id},
					{cardset_id: {$in: cardset.cardGroups}}
				]
			};
			Cards._collection.remove(filterQuery);
			Meteor.subscribe("previewCards", cardset._id);
		}
	}
});

Template.cardsetPreview.onDestroyed(function () {
	if (Router.current().params._id) {
		Meteor.subscribe('cardsetCards', Router.current().params._id);
		Session.set('activeCard', undefined);
	}
});

/*
 * ############################################################################
 * cardsetList
 * ############################################################################
 */
Template.cardsetList.helpers({
	isShuffledCardset: function () {
		if (Router.current().route.getName() === "demolist") {
			return Cardsets.findOne({kind: 'demo', name: "DemoCardset", shuffled: true}).shuffled;
		} else if (Router.current().route.getName() === "makinglist") {
			return Cardsets.findOne({kind: 'demo', name: "MakingOfCardset", shuffled: true}).shuffled;
		} else {
			return Cardsets.findOne({_id: Router.current().params._id}).shuffled;
		}
	},
	cardsetList: function () {
		let isDemo = (Router.current().route.getName() === "demolist" || Router.current().route.getName() === "makinglist");
		if (Router.current().route.getName() === "cardsetlistid" || Router.current().route.getName() === "presentationlist" || isDemo) {
			let cardsetId = Router.current().params._id;
			if (isDemo) {
				if (Route.isDemo()) {
					cardsetId = Cardsets.findOne({kind: 'demo', name: "DemoCardset", shuffled: true})._id;
				} else {
					cardsetId = Cardsets.findOne({kind: 'demo', name: "MakingOfCardset", shuffled: true})._id;
				}
			}
			if (this.shuffled) {
				let cardsetFilter = [];
				let sortCardsets = Cardsets.find({_id: {$in: this.cardGroups}}, {
					sort: {name: 1}, fields: {_id: 1, name: 1, kind: 1, owner: 1}
				}).fetch();
				sortCardsets.forEach(function (cardset) {
					if (cardset._id !== cardsetId) {
						cardsetFilter.push(cardset);
					}
				});
				return cardsetFilter;
			} else {
				return Cardsets.find({_id: this._id}).fetch();
			}
		} else {
			return Cardsets.find({_id: Session.get('tempLearningIndex')}).fetch();
		}
	},
	getPriority: function (index) {
		return index + 1;
	},
	cleanContent: function (text) {
		return CardVisuals.removeMarkdeepTags(text);
	},
	gotCards: function () {
		if (Router.current().route.getName() === "cardsetlistid" || Router.current().route.getName() === "presentationlist" || Router.current().route.getName() === "demolist" || Router.current().route.getName() === "makinglist") {
			if (this.shuffled) {
				return Cards.find({cardset_id: {$in: this.cardGroups}}).count();
			} else {
				return Cards.find({cardset_id: this._id}).count();
			}
		} else {
			return Cards.find({cardset_id: Session.get('tempLearningIndex'), cardType: 0}).count();
		}
	},
	cardSubject: function () {
		if (Router.current().route.getName() === "cardsetlistid" || Router.current().route.getName() === "presentationlist" || Router.current().route.getName() === "demolist" || Router.current().route.getName() === "makinglist") {
			return _.uniq(Cards.find({
				cardset_id: this._id
			}, {
				cardset_id: 1,
				subject: 1,
				sort: {subject: 1}
			}).fetch(), function (card) {
				return card.subject;
			});
		} else {
			return _.uniq(Cards.find({
				cardset_id: this._id
			}, {
				cardset_id: 1,
				subject: 1,
				sort: {subject: 1}
			}).fetch(), function (card) {
				return card.subject;
			});
		}
	},
	cardList: function (countCards) {
		let sortQuery;
		sortQuery = CardType.getSortQuery(Cardsets.findOne({_id: this.cardset_id}).cardType);
		if (countCards) {
			return Cards.find({
				cardset_id: this.cardset_id,
				subject: this.subject
			}, {
				fields: {
					_id: 1,
					front: 1,
					back: 1,
					hint: 1,
					lecture: 1,
					top: 1,
					bottom: 1,
					cardset_id: 1
				},
				sort: sortQuery
			}).count();
		}
		let cards = Cards.find({
			cardset_id: this.cardset_id,
			subject: this.subject
		}, {
			fields: {
				_id: 1,
				front: 1,
				back: 1,
				hint: 1,
				lecture: 1,
				top: 1,
				bottom: 1,
				cardset_id: 1
			},
			sort: sortQuery
		}).fetch();
		return CardVisuals.setTypeAndDifficulty(cards);
	},
	getColors: function () {
		switch (this.kind) {
			case "personal":
				return "btn-warning";
			case "free":
				return "btn-info";
			case "edu":
				return "btn-success";
			case "pro":
				return "btn-danger";
			case "demo":
				return "btn-demo";
		}
	},
	gotReferences: function () {
		return Cardsets.findOne({_id: Router.current().params._id}).cardGroups !== [""];
	},
	getText: function () {
		let cubeSides = CardType.getCardTypeCubeSides(this.cardType);
		switch (cubeSides[0].contentId) {
			case 1:
				return this.front;
			case 2:
				return this.back;
			case 3:
				return this.hint;
			case 4:
				return this.lecture;
			case 5:
				return this.top;
			case 6:
				return this.bottom;
		}
	}
});

Template.cardsetList.events({
	'click .cardListRow': function (evt) {
		let cubeSides = CardType.getCardTypeCubeSides($(evt.target).data('card-type'));
		Session.set('cardType', $(evt.target).data('card-type'));
		Session.set('activeCardContentId', cubeSides[0].contentId);
		if (Router.current().route.getName() === "cardsetlistid" || Router.current().route.getName() === "presentationlist" || Router.current().route.getName() === "demolist" || Router.current().route.getName() === "makinglist") {
			Session.set('activeCard', $(evt.target).data('id'));
			if (Router.current().route.getName() === "presentationlist") {
				Router.go('presentation', {
					_id: Router.current().params._id
				});
			} else if (Router.current().route.getName() === "demolist") {
				Router.go('demo');
			} else if (Router.current().route.getName() === "makinglist") {
				Router.go('making');
			} else {
				Router.go('cardsetcard', {
					_id: Router.current().params._id,
					card_id: $(evt.target).data('id')
				});
			}
		} else {
			let learningUnit = $(evt.target).data('id');
			Session.set('learningIndex', Session.get('tempLearningIndex'));
			Session.set('learningUnit', learningUnit);
			Session.set('subject', Cards.findOne({_id: learningUnit}).subject);
			$('#showSelectLearningUnitModal').modal('hide');
		}
	}
});
/*
 * ############################################################################
 * cardsetInfo
 * ############################################################################
 */

Template.cardsetInfo.onCreated(function () {
	$('[data-toggle="tooltip"]').tooltip({
		container: 'body'
	});
});

Template.cardsetInfo.onRendered(function () {
	$('[data-toggle="tooltip"]').tooltip({
		container: 'body'
	});
	new ResizeSensor($('#cardsetInfoDetail'), function () {
		CardsetVisuals.resizeCardsetInfo();
	});
	CardsetVisuals.resizeCardsetInfo();
});

Template.cardsetInfo.helpers({
	getStatus: function () {
		if (this.visible) {
			switch (this.kind) {
				case "free":
					return TAPi18n.__('access-level.free.short');
				case "edu":
					return TAPi18n.__('access-level.edu.short');
				case "pro":
					return TAPi18n.__('access-level.pro.short');
				case "personal":
					return TAPi18n.__('access-level.private.short');
			}
		} else {
			if (this.kind === 'pro' && this.request === true) {
				return TAPi18n.__('sidebar-nav.review');
			} else {
				return TAPi18n.__('access-level.private.short');
			}
		}
	},
	canPublish: function () {
		let cardCount;
		if (this.shuffled) {
			cardCount = 2;
		} else {
			cardCount = 1;
		}
		return (this.quantity >= cardCount || this.reviewed || this.request);
	},
	isLecturerAndHasRequest: function () {
		return (Roles.userIsInRole(Meteor.userId(), 'lecturer') && this.request === true && this.owner !== Meteor.userId());
	},
	gotPresentation: function () {
		if (this.shuffled) {
			return true;
		} else {
			return CardType.gotPresentationMode(this.cardType);
		}
	},
	gotEnoughCardsToFilter: function () {
		return (this.quantity >= 3);
	},
	gotLearningModes: function () {
		if (this.shuffled) {
			for (let i = 0; i < this.cardGroups.length; i++) {
				if (CardType.gotLearningModes(Cardsets.findOne(this.cardGroups[i]).cardType)) {
					return true;
				}
			}
		} else {
			return CardType.gotLearningModes(this.cardType);
		}
	},
	learningLeitner: function () {
		return Leitner.findOne({cardset_id: Router.current().params._id, user_id: Meteor.userId()});
	},
	learningMemo: function () {
		return Wozniak.findOne({
			cardset_id: Router.current().params._id,
			user_id: Meteor.userId()});
	},
	learning: function () {
		return (Leitner.findOne({
			cardset_id: Router.current().params._id,
			user_id: Meteor.userId()
		}) || Wozniak.findOne({
			cardset_id: Router.current().params._id,
			user_id: Meteor.userId(),
			interval: {$ne: 0}
		}));
	},
	enableIfPublished: function () {
		return this.kind !== 'personal';
	},
	isInBonus: function () {
		return Bonus.isInBonus(Session.get('activeCardset')._id, Meteor.userId());
	}
});

Template.cardsetInfo.events({
	'click #exportCardsBtn': function () {
		let name = this.name;
		Meteor.call('exportCards', this._id, function (error, result) {
			if (error) {
				BertAlertVisuals.displayBertAlert(TAPi18n.__('export.cards.failure'), 'danger', 'growl-top-left');
			} else {
				let exportData = new Blob([result], {
					type: "application/json"
				});
				saveAs(exportData, TAPi18n.__('export.filename.export') + "_" + TAPi18n.__('export.filename.cards') + "_" + name + moment().format('_YYYY_MM_DD') + ".json");
			}
		});
	},
	'click #editShuffle': function () {
		Router.go('editshuffle', {_id: Router.current().params._id});
	},
	'click #importCardsBtn': function () {
		Session.set('importType', 1);
	},
	"click #startPresentation": function () {
		Router.go('presentation', {_id: this._id});
	},
	"click #learnBox": function () {
		addToLeitner(this._id);
		Router.go('box', {
			_id: this._id
		});
	},
	"click #learnMemo": function () {
		Meteor.call("addWozniakCards", this._id);
		Router.go('memo', {
			_id: this._id
		});
	},
	"click #leitnerProgress": function () {
		Router.go('progress', {
			_id: this._id,
			user_id: Meteor.userId()
		});
	},
	"click #showStats": function () {
		Router.go('cardsetstats', {_id: Router.current().params._id});
	},
	"click #manageEditors": function () {
		Router.go('cardseteditors', {_id: Router.current().params._id});
	},
	"click #startBonus": function () {
		Session.set('isNewBonus', true);
	},
	"click #manageBonus": function () {
		Session.set('isNewBonus', false);
	},
	"click #collapseManageButton": function () {
		changeCollapseIcon("#collapseMangeIcon");
	},
	"click #leaveCardsetButton": function () {
		Router.go('pool');
	},
	"click .bonusBtn": function () {
		if (this.kind === "personal") {
			let cardCount;
			if (this.shuffled) {
				cardCount = 2;
			} else {
				cardCount = 1;
			}
			let cardCountMessage = "";
			if (cardCount === 1) {
				cardCountMessage = cardCount + " " + TAPi18n.__('confirmLearn-form.card');
			} else {
				cardCountMessage = cardCount + " " + TAPi18n.__('confirmLearn-form.cards');
			}
			Bert.defaults.hideDelay = 15000;
			BertAlertVisuals.displayBertAlert(TAPi18n.__('bertAlert.publishBonus.message', {cardcount: cardCountMessage, edu: Meteor.settings.public.university.default}), 'warning', 'growl-top-left');
		}
	},
	'click a': function (evt) {
		CardNavigation.linkNavigation(evt);
	}
});

/*
 * ############################################################################
 * cardsetInfoBox
 * ############################################################################
 */

Template.cardsetInfoBox.onRendered(function () {
	$('[data-toggle="tooltip"]').tooltip({
		container: 'body'
	});
});

Template.cardsetInfoBox.helpers({
	getColors: function () {
		switch (this.kind) {
			case "personal":
				return "btn-warning";
			case "free":
				return "btn-info";
			case "edu":
				return "btn-success";
			case "pro":
				return "btn-danger";
			case "demo":
				return "btn-demo";
		}
	},
	getName: function () {
		let shuffled = "";
		if (this.shuffled) {
			shuffled = TAPi18n.__('admin.shuffled') + " ";
		}
		return shuffled;
	}
});


/*
 * ############################################################################
 * cardsetInfoBoxContentOne
 * ############################################################################
 */

Template.cardsetInfoBoxContentOne.helpers({
	canViewForFree: function () {
		return (this.kind === "edu" && (Roles.userIsInRole(Meteor.userId(), ['university', 'lecturer'])));
	},
	ratingEnabled: function () {
		return this.ratings === true;
	},
	canRateCardset: function () {
		let result = Cardsets.findOne({_id: this._id}, {fields: {owner: 1}});
		if (result !== undefined) {
			return result.owner !== Meteor.userId();
		} else {
			return false;
		}
	},
	getAverageRating: function () {
		let ratings = Ratings.find({cardset_id: this._id}).fetch();
		let averageRating = 0;
		for (let i = 0; i < ratings.length; i++) {
			averageRating += ratings[i].rating;
		}
		return averageRating / ratings.length;
	},
	getUserRating: function () {
		var userrating = Ratings.findOne({
			cardset_id: this._id,
			user: Meteor.userId()
		});
		if (userrating) {
			return userrating.rating;
		} else {
			return 0 + " " + TAPi18n.__('cardset.info.notRated');
		}
	},
	gotOriginalAuthorData: function () {
		if (this.originalAuthorName !== undefined) {
			return (this.originalAuthorName.birthname !== undefined || this.originalAuthorName.legacyName !== undefined);
		} else {
			return "";
		}
	},
	hasAmount: function () {
		return this.kind === 'pro' || this.kind === 'edu';
	},
	isPurchased: function () {
		return Paid.findOne({cardset_id: this._id}) !== undefined;
	},
	getDateOfPurchase: function () {
		return moment(Paid.findOne({cardset_id: this._id}).date).locale(Session.get('activeLanguage')).format('LL');
	},
	getReviewer: function () {
		var reviewer = Meteor.users.findOne(this.reviewer);
		return (reviewer !== undefined) ? reviewer.profile.name : undefined;
	},
	getCardType: function () {
		return CardType.getCardTypeName(this.cardType);
	},
	gotNotesForDifficultyLevel: function () {
		return CardType.gotNotesForDifficultyLevel(this.cardType);
	},
	getDifficultyName: function () {
		if (CardType.gotNotesForDifficultyLevel(this.cardType)) {
			return TAPi18n.__('difficultyNotes' + this.difficulty);
		} else {
			if (!CardType.gotDifficultyLevel(this.cardType)) {
				return TAPi18n.__('difficulty0');
			} else {
				return TAPi18n.__('difficulty' + this.difficulty);
			}
		}
	},
	getLearningMode: function () {
		let actualDate = new Date(new Date().getTime() + 24 * 60 * 60 * 1000);
		actualDate.setHours(0, 0, 0, 0);
		let count = 0;
		if (Leitner.find({cardset_id: this._id, user_id: Meteor.userId(), active: true}).count()) {
			count += 1;
		}
		if (Wozniak.find({
			cardset_id: this._id, user_id: Meteor.userId(), nextDate: {
				$lte: actualDate
			}
		}).count()) {
			count += 2;
		}
		switch (count) {
			case 0:
				return TAPi18n.__('set-list.none');
			case 1:
				return TAPi18n.__('set-list.leitner');
			case 2:
				return TAPi18n.__('set-list.wozniak');
			case 3:
				return TAPi18n.__('set-list.both');
		}
	}
});

Template.cardsetInfoBoxContentOne.events({
	'click #rating': function () {
		var cardset_id = Template.parentData(1)._id;
		var rating = $('#rating').data('userrating');
		var count = Ratings.find({
			cardset_id: cardset_id,
			user: Meteor.userId()
		}).count();
		if (count === 0) {
			Meteor.call("addRating", cardset_id, rating);
		} else {
			Meteor.call("updateRating", cardset_id, rating);
		}
	},
	'click .showLicense': function (event) {
		event.preventDefault();
		Session.set('selectedCardset', $(event.target).data('id'));
	}
});

/*
 * ############################################################################
 * cardsetInfoBoxContentTwo
 * ############################################################################
 */

Template.cardsetInfoBoxContentTwo.helpers({
	canViewForFree: function () {
		return (this.kind === "edu" && (Roles.userIsInRole(Meteor.userId(), ['university', 'lecturer'])));
	},
	ratingEnabled: function () {
		return this.ratings === true && Session.get('ratingsLoaded');
	},
	hasRated: function () {
		var count = Ratings.find({
			cardset_id: this._id,
			user: Meteor.userId()
		}).count();
		var cardset = Cardsets.findOne(this._id);
		if (cardset !== null) {
			return count !== 0;
		}
	},
	canRateCardset: function () {
		let result = Cardsets.findOne({_id: this._id}, {fields: {owner: 1}});
		if (result !== undefined) {
			return result.owner !== Meteor.userId();
		} else {
			return false;
		}
	},
	getAverageRating: function () {
		let ratings = Ratings.find({cardset_id: this._id}).fetch();
		let averageRating = 0;
		for (let i = 0; i < ratings.length; i++) {
			averageRating += ratings[i].rating;
		}
		return averageRating / ratings.length;
	},
	getUserRating: function () {
		var userrating = Ratings.findOne({
			cardset_id: this._id,
			user: Meteor.userId()
		});
		if (userrating) {
			return userrating.rating;
		} else {
			return 0 + " " + TAPi18n.__('cardset.info.notRated');
		}
	},
	hasAmount: function () {
		return this.kind === 'pro' || this.kind === 'edu';
	},
	isPurchased: function () {
		return Paid.findOne({cardset_id: this._id}) !== undefined;
	},
	getDateOfPurchase: function () {
		return moment(Paid.findOne({cardset_id: this._id}).date).locale(Session.get('activeLanguage')).format('LL');
	},
	getReviewer: function () {
		var reviewer = Meteor.users.findOne(this.reviewer);
		return (reviewer !== undefined) ? reviewer.profile.name : undefined;
	},
	getCardType: function () {
		return CardType.getCardTypeName(this.cardType);
	},
	gotNotesForDifficultyLevel: function () {
		return CardType.gotNotesForDifficultyLevel(this.cardType);
	},
	getDifficultyName: function () {
		if (CardType.gotNotesForDifficultyLevel(this.cardType)) {
			return TAPi18n.__('difficultyNotes' + this.difficulty);
		} else {
			return TAPi18n.__('difficulty' + this.difficulty);
		}
	}
});

Template.cardsetInfoBoxContentTwo.events({
	'click #rating': function () {
		var cardset_id = Template.parentData(1)._id;
		var rating = $('#rating').data('userrating');
		var count = Ratings.find({
			cardset_id: cardset_id,
			user: Meteor.userId()
		}).count();
		if (count === 0) {
			Meteor.call("addRating", cardset_id, rating);
		} else {
			Meteor.call("updateRating", cardset_id, rating);
		}
	}
});

/*
 * ############################################################################
 * leaveLearnPhaseForm
 * ############################################################################
 */

Template.leaveLearnPhaseForm.events({
	'click #leaveLearnPhaseConfirm': function () {
		var id = Router.current().params._id;


		$('#leaveModal').modal('hide');
		$('body').removeClass('modal-open');
		$('.modal-backdrop').remove();
		$('#leaveModal').on('hidden.bs.modal', function () {
			Meteor.call("deleteLeitner", id);
			Router.go('home');
		});
	}
});

/*
 * ############################################################################
 * leaveCardsetForm
 * ############################################################################
 */

Template.leaveCardsetForm.events({
	'click #leaveCardsetConfirm': function () {
		var id = Router.current().params._id;


		$('#leaveCardsetModal').modal('hide');
		$('body').removeClass('modal-open');
		$('.modal-backdrop').remove();
		$('#leaveCardsetModal').on('hidden.bs.modal', function () {
			Meteor.call("deleteLeitner", id);
			Meteor.call("deleteWozniak", id);
			Router.go('home');
		});
	}
});


/*
 * ############################################################################
 * leaveEditorsForm
 * ############################################################################
 */

Template.leaveEditorsForm.events({
	'click #leaveEditorsConfirm': function () {
		$('#leaveEditorsModal').modal('hide');
		$('body').removeClass('modal-open');
		$('.modal-backdrop').remove();
		$('#leaveEditorsModal').on('hidden.bs.modal', function () {
			Meteor.call("leaveEditors", Router.current().params._id);
		});
	}
});

/*
* ############################################################################
* chooseFlashcards
* ############################################################################
*/

Template.chooseFlashcards.created = function () {
	let chooseFlashcardsFilter = [];
	chooseFlashcardsFilter[0] = [];
	chooseFlashcardsFilter[1] = [0, 1, 2, 3, 4, 5];
	Session.set('chooseFlashcardsFilter', chooseFlashcardsFilter);
};

Template.chooseFlashcards.helpers({
	getCardCount: function (category, item) {
		let cardsetFilter = Router.current().params._id;
		if (this.shuffled) {
			cardsetFilter = {$in: this.cardGroups};
		}
		if (category === 0) {
			return 0;
		} else if (category === 1) {
			return Cards.find({
				cardset_id: cardsetFilter,
				learningGoalLevel: item
			}).count();
		} else {
			let chooseFlashcardsFilter = Session.get('chooseFlashcardsFilter');
			if ((chooseFlashcardsFilter[1].length) === 0) {
				return 0;
			}
			let learningGoalLevelFilter = {$ne: null};
			if (chooseFlashcardsFilter[1].length) {
				learningGoalLevelFilter = {$in: chooseFlashcardsFilter[1]};
			}
			return Cards.find({
				cardset_id: cardsetFilter,
				learningGoalLevel: learningGoalLevelFilter
			}).count();
		}
	},
	gotLearningGoal: function () {
		return CardType.gotLearningGoal(this.cardType);
	},
	getSortMode: function () {
		let chooseFlashcardsFilter = Session.get('chooseFlashcardsFilter');
		if (chooseFlashcardsFilter[0] === 0) {
			return TAPi18n.__('filter-cards.sortMode0');
		} else {
			return TAPi18n.__('filter-cards.sortMode1');
		}
	},
	isPresentationMode: function () {
		return Session.get('chooseFlashcardsMode');
	}
});

Template.chooseFlashcards.events({
	"click #createCardFilter": function () {
		$('#chooseFlashcardsModal').modal('hide');
		$('body').removeClass('modal-open');
		$('.modal-backdrop').remove();
	},
	"click .sortFilter": function () {
		let chooseFlashcardsFilter = Session.get('chooseFlashcardsFilter');
		if (chooseFlashcardsFilter[0] === 0) {
			chooseFlashcardsFilter[0] = 1;
		} else {
			chooseFlashcardsFilter[0] = 0;
		}
		Session.set('chooseFlashcardsFilter', chooseFlashcardsFilter);
	}
});

Template.chooseFlashcards.onRendered(function () {
	$('#chooseFlashcardsModal').on('hidden.bs.modal', function () {
		let chooseFlashcardsFilter = [""];
		chooseFlashcardsFilter[0] = [];
		chooseFlashcardsFilter[1] = [0, 1, 2, 3, 4, 5];
		Session.set('chooseFlashcardsFilter', chooseFlashcardsFilter);
	});
});

/*
* ############################################################################
* chooseFlashcardsButton
* ############################################################################
*/

Template.chooseFlashcardsButton.helpers({
	inFlashcardFilterSelection: function (category, item) {
		return Session.get('chooseFlashcardsFilter')[category].includes(item);
	}
});

Template.chooseFlashcardsButton.events({
	"click .addCardFilter": function (event) {
		let category = $(event.target).data('category');
		let chooseFlashcardsFilter = Session.get('chooseFlashcardsFilter');
		let item = $(event.target).data('item');
		chooseFlashcardsFilter[category].push(item);
		Session.set('chooseFlashcardsFilter', chooseFlashcardsFilter);
	},
	"click .removeCardFilter": function (event) {
		let category = $(event.target).data('category');
		let chooseFlashcardsFilter = Session.get('chooseFlashcardsFilter');
		let item = $(event.target).data('item');
		let pos = chooseFlashcardsFilter[category].indexOf(item);
		chooseFlashcardsFilter[category].splice(pos, 1);
		Session.set('chooseFlashcardsFilter', chooseFlashcardsFilter);
	}
});
/*
* ############################################################################
* cardsetInfoBox
* ############################################################################
*/
Template.cardsetInfoBox.events({
	"click #collapseCardsetInfoButton": function () {
		changeCollapseIcon("#collapseCardsetInfoIcon");
	}
});

/*
* ############################################################################
* cardsetLearnActivityStatistic
* ############################################################################
*/

Template.cardsetLearnActivityStatistic.onRendered(function () {
	Session.set('activeCardset', Cardsets.findOne({_id: Router.current().params._id}));
});

Template.cardsetLearnActivityStatistic.helpers({
	getCardsetStats: function () {
		return Session.get("learnerStats");
	},
	getPercentage: function (count) {
		let percentage = Math.round(count / Session.get('activeCardset').quantity * 100);
		if (percentage > 0) {
			return '<span class="cardPercentage">[' + percentage + ' %]</span>';
		}
	},
	earnedTrophy: function () {
		let totalCards = this.box1 + this.box2 + this.box3 + this.box4 + this.box5 + this.box6;
		let box6Percentage = (this.box6 / totalCards) * 100;
		return box6Percentage >= 95;
	}
});

Template.cardsetLearnActivityStatistic.events({
	"click #exportCSV": function () {
		var cardset = Cardsets.findOne({_id: this._id});
		var hiddenElement = document.createElement('a');
		var header = [];
		header[0] = TAPi18n.__('leitnerProgress.box', {number: 1});
		header[1] = TAPi18n.__('leitnerProgress.box', {number: 2});
		header[2] = TAPi18n.__('leitnerProgress.box', {number: 3});
		header[3] = TAPi18n.__('leitnerProgress.box', {number: 4});
		header[4] = TAPi18n.__('leitnerProgress.box', {number: 5});
		header[5] = TAPi18n.__('leitnerProgress.learned');
		header[6] = TAPi18n.__('box_export_birth_name');
		header[7] = TAPi18n.__('box_export_given_name');
		header[8] = TAPi18n.__('box_export_mail');
		header[9] = TAPi18n.__('leitnerProgress.percentage');
		Meteor.call("getCSVExport", cardset._id, header, function (error, result) {
			if (error) {
				throw new Meteor.Error(error.statusCode, 'Error could not receive content for .csv');
			}
			if (result) {
				var statistics = TAPi18n.__('box_export_statistics');
				hiddenElement.href = 'data:text/csv;charset=utf-8,%EF%BB%BF' + encodeURIComponent(result);
				hiddenElement.target = '_blank';
				var str = (cardset.name + "_" + statistics + "_" + new Date() + ".csv");
				hiddenElement.download = str.replace(/ /g, "_").replace(/:/g, "_");
				document.body.appendChild(hiddenElement);
				hiddenElement.click();
			}
		});
	},
	"click #backButton": function () {
		Router.go('cardsetdetailsid', {_id: this._id});
	},
	"click .detailed-stats": function (event) {
		Router.go('progress', {
			_id: Router.current().params._id,
			user_id: $(event.target).data('id')
		});
	},
	"click #showIntervalHelp": function (event) {
		event.stopPropagation();
		Session.set('helpFilter', "leitner");
		Router.go('help');
	}
});

Template.cardsetLearnActivityStatistic.created = function () {
	Session.set("learnerStats", "");
	Meteor.call("getLearningData", Router.current().params._id, function (error, result) {
		if (error) {
			throw new Meteor.Error(error.statusCode, 'Error could not receive content for stats');
		}
		if (result) {
			Session.set("learnerStats", result);
		}
	});
};

/*
* ############################################################################
* cardsetManageEditors
* ############################################################################
*/

Template.cardsetManageEditors.helpers({
	getEditors: function () {
		return Session.get("editorsList");
	}
});

Template.cardsetManageEditors.events({
	"click #backButton": function () {
		Router.go('cardsetdetailsid', {_id: Router.current().params._id});
	},
	"click .addEditor": function (event) {
		Meteor.call("addEditor", Router.current().params._id, $(event.target).data('id'));
	},
	"click .removeEditor": function (event) {
		Meteor.call("removeEditor", Router.current().params._id, $(event.target).data('id'));
	}
});

Template.cardsetManageEditors.created = function () {
	Session.set("editorsList", "");
	Meteor.call("getEditors", Router.current().params._id, function (error, result) {
		if (error) {
			throw new Meteor.Error(error.statusCode, 'Error could not receive content for editors');
		}
		if (result) {
			Session.set("editorsList", result);
		}
	});
};

/*
 * ############################################################################
 * cardsetEndLearnForm
 * ############################################################################
 */

Template.cardsetEndLearnForm.events({
	"click #confirmEndLearn": function () {
		if (Cardsets.findOne(Router.current().params._id).learningActive) {
			Meteor.call("deactivateBonus", Router.current().params._id);
		}
		$('#confirmEndLearnModal').modal('hide');
		$('body').removeClass('modal-open');
		$('.modal-backdrop').remove();
	}
});

Session.setDefault('importType', 1);
/*
 * ############################################################################
 * cardsetImportForm
 * ############################################################################
 */

Template.cardsetImportForm.onCreated(function () {
	Template.instance().uploading = new ReactiveVar(false);
});

Template.cardsetImportForm.onRendered(function () {
	$('#importModal').on('hidden.bs.modal', function () {
		$('#uploadError').html('');
	});
});

Template.cardsetImportForm.helpers({
	uploading: function () {
		return Template.instance().uploading.get();
	},
	importType: function (importType) {
		return Session.get('importType') === importType;
	}
});

Template.cardsetImportForm.events({
	"click #importType1": function () {
		Session.set('importType', 1);
	},
	"click #importType2": function () {
		Session.set('importType', 2);
	},
	'change [name="uploadFile"]': function (evt, tmpl) {
		tmpl.uploading.set(true);
		let cardset_id = Template.parentData(1)._id;
		let importType = Session.get('importType');
		if (importType === 1) {
			if (evt.target.files[0].name.match(/\.(json)$/)) {
				var reader = new FileReader();
				reader.onload = function () {
					try {
						let res;
						if (this.result.charAt(0) === '[' && this.result.charAt(this.result.length - 1) === ']') {
							res = $.parseJSON(this.result);
						} else {
							res = $.parseJSON('[' + this.result + ']');
						}
						Meteor.call('importCards', res, cardset_id, Number(importType), function (error) {
							if (error) {
								tmpl.uploading.set(false);
								BertAlertVisuals.displayBertAlert(TAPi18n.__('import.failure'), 'danger', 'growl-top-left');
							} else {
								tmpl.uploading.set(false);
								Session.set('activeCard', undefined);
								BertAlertVisuals.displayBertAlert(TAPi18n.__('import.success.cards'), 'success', 'growl-top-left');
								$('#importModal').modal('toggle');
							}
						});
					} catch (e) {
						tmpl.uploading.set(false);
						BertAlertVisuals.displayBertAlert(TAPi18n.__('import.failure'), 'danger', 'growl-top-left');
					}
				};
				reader.readAsText(evt.target.files[0]);
			} else if (evt.target.files[0].name.match(/\.(csv)$/)) {
				Papa.parse(evt.target.files[0], {
					header: true,
					complete: function (results) {
						Meteor.call('importCards', results.data, cardset_id, Number(importType), function (error) {
							if (error) {
								tmpl.uploading.set(false);
								BertAlertVisuals.displayBertAlert(TAPi18n.__('import.failure'), 'danger', 'growl-top-left');
							} else {
								tmpl.uploading.set(false);
								BertAlertVisuals.displayBertAlert(TAPi18n.__('import.success.cards'), 'success', 'growl-top-left');
								$('#importModal').modal('toggle');
							}
						});
					}
				});
			} else {
				tmpl.uploading.set(false);
				$('#uploadError').html('<br><div class="alert alert-danger" role="alert">' + TAPi18n.__('upload-form.wrong-file') + '</div>');
			}
		} else {
			tmpl.uploading.set(false);
			$('#importModal').modal('toggle');
		}
	}
});

/*
 * ############################################################################
 * cardsetConfirmForm
 * ############################################################################
 */

Template.cardsetConfirmForm.events({
	'click #cardDelete': function () {
		var id = Session.get('cardId');

		$('#confirmModal').on('hidden.bs.modal', function () {
			Meteor.call("deleteCard", id);
		}).modal('hide');
	}
});

/*
 * ############################################################################
 * cardsetPublishForm
 * ############################################################################
 */

Template.cardsetPublishForm.onRendered(function () {
	$('#publishModal').on('hidden.bs.modal', function () {
		var cardset = Cardsets.findOne(Router.current().params._id);

		$('#publishKind > label').removeClass('active');
		$('#publishKind > label > input').filter(function () {
			return this.value === cardset.kind;
		}).parent().addClass('active');

		$('#publishKind > label > input').filter(function () {
			return this.value === cardset.kind;
		}).prop('checked', true);

		$('#publishPrice').val(cardset.price);
	});
});

Template.cardsetPublishForm.events({
	'shown.bs.modal #publishModal': function () {
		Session.set('kindWithPrice', false);
	},
	'hidden.bs.modal #publishModal': function () {
		Session.set('kindWithPrice', false);
	},
	'click #cardsetPublish': function (evt, tmpl) {
		let kind = tmpl.find('#publishKind > .active > input').value;
		let price = 0;
		let visible = true;
		let license = [];

		if (kind === 'edu' || kind === 'pro') {
			if (tmpl.find('#publishPrice') !== null) {
				price = tmpl.find('#publishPrice').value;
			} else {
				price = this.price;
			}
		}
		if (kind === 'personal') {
			visible = false;
			Meteor.call('updateLicense', Router.current().params._id, license);
		}
		if (kind === 'pro') {
			visible = false;
			Meteor.call("makeProRequest", Router.current().params._id);

			let text = "Kartensatz " + this.name + " zur Überprüfung freigegeben";
			let type = "Kartensatz-Freigabe";
			let target = "lecturer";

			Meteor.call("addNotification", target, type, text, Router.current().params._id, "lecturer");

			license.push("by");
			license.push("nd");
			Meteor.call("updateLicense", Router.current().params._id, license);
			BertAlertVisuals.displayBertAlert(TAPi18n.__('cardset.request.alert'), 'success', 'growl-top-left');
		}

		Meteor.call("publishCardset", Router.current().params._id, kind, price, visible);
		$('#publishModal').modal('hide');
	}
});

/*
 * ############################################################################
 * publishKind
 * ############################################################################
 */

Template.publishKind.helpers({
	kindWithPrice: function () {
		return Session.get('kindWithPrice');
	},
	kindIsActive: function (kind) {
		if (kind === 'personal' && this.kind === undefined) {
			return true;
		} else {
			return kind === this.kind;
		}
	},
	priceIsSelected: function (price) {
		return price === this.price ? 'selected' : '';
	}
});

Template.publishKind.events({
	'change #publishKind': function () {
		var kind = $('#publishKind input[name=kind]:checked').val();
		var kindWithPrice = (kind === 'edu' || kind === 'pro');
		Session.set('kindWithPrice', kindWithPrice);
	}
});

/*
 * ############################################################################
 * deleteCardsForm
 * ############################################################################
 */

Template.deleteCardsForm.events({
	'click #deleteCardsConfirm': function () {
		var id = this._id;
		$('#deleteCardsModal').on('hidden.bs.modal', function () {
			Meteor.call("deleteCards", id);
		}).modal('hide');
	}
});

/*
 * ############################################################################
 * selectLicenseForm
 * ############################################################################
 */

Template.selectLicenseForm.onRendered(function () {
	$('#selectLicenseModal').on('hidden.bs.modal', function () {
		var cardset = Cardsets.findOne(Router.current().params._id);
		var license = cardset.license;

		$('#cc-modules > label').removeClass('active');
		$('#modulesLabel').css('color', '');
		$('#helpCC-modules').html('');

		if (license.includes('by')) {
			$('#cc-option0').addClass('active');
		}
		if (license.includes('nc')) {
			$('#cc-option1').addClass('active');
		}
		if (license.includes('nd')) {
			$('#cc-option2').addClass('active');
		}
		if (license.includes('sa')) {
			$('#cc-option3').addClass('active');
		}
	});
});

Template.selectLicenseForm.events({
	'click #licenseSave': function () {
		if ($("#cc-option2").hasClass('active') && $("#cc-option3").hasClass('active') || $("#cc-modules").children().hasClass('active') && !($("#cc-option0").hasClass('active'))) {
			$('#modulesLabel').css('color', '#b94a48');
			$('#helpCC-modules').html(TAPi18n.__('modal-dialog.wrongCombination'));
			$('#helpCC-modules').css('color', '#b94a48');
		} else {
			var license = [];

			if ($("#cc-option0").hasClass('active')) {
				license.push("by");
			}
			if ($("#cc-option1").hasClass('active')) {
				license.push("nc");
			}
			if ($("#cc-option2").hasClass('active')) {
				license.push("nd");
			}
			if ($("#cc-option3").hasClass('active')) {
				license.push("sa");
			}

			Meteor.call('updateLicense', this._id, license);
			$('#selectLicenseModal').modal('hide');
		}
	},
	'change #cc-modules': function () {
		$('#modulesLabel').css('color', '');
		$('#helpCC-modules').html('');
	}
});

Template.selectLicenseForm.helpers({
	licenseIsActive: function (license) {
		var cardset = Cardsets.findOne(Router.current().params._id);
		if (cardset !== undefined) {
			var licenses = cardset.license;

			if (licenses.includes(license)) {
				return true;
			}
		} else {
			return null;
		}
	}
});

/*
 * ############################################################################
 * reportCardsetForm
 * ############################################################################
 */

Template.reportCardsetForm.onRendered(function () {
	$('#reportModal').on('hidden.bs.modal', function () {
		$('#helpReportCardsetText').html('');
		$('#reportCardsetTextLabel').css('color', '');
		$('#reportCardsetText').css('border-color', '');
		$('#reportCardsetText').val('');
		$('#reportCardsetReason').val($('#reportCardsetReason option:first').val());
	});
});
Template.reportCardsetForm.events({
	'click #reportCardsetSave': function () {
		if ($('#reportCardsetText').val().length < 50) {
			$('#reportCardsetTextLabel').css('color', '#b94a48');
			$('#reportCardsetText').css('border-color', '#b94a48');
			$('#helpReportCardsetText').html(TAPi18n.__('modal-dialog.text_chars'));
			$('#helpReportCardsetText').css('color', '#b94a48');
		} else {
			var text = $('#reportCardsetText').val();
			var type;
			var link_id;

			if ($('#reportCardsetReason').val() === "Benutzer melden" || $('#reportCardsetReason').val() === "Report user") {
				type = "Gemeldeter Benutzer";
				link_id = this.owner;
			} else {
				type = "Gemeldeter Kartensatz";
				link_id = this._id;
			}

			var target = "admin";

			Meteor.call("addNotification", target, type, text, link_id, target);
			$('#reportModal').modal('hide');
		}
	},
	'keyup #reportCardsetText': function () {
		$('#reportCardsetTextLabel').css('color', '');
		$('#reportCardsetText').css('border-color', '');
		$('#helpReportCardsetText').html('');
	}
});

/*
 * ############################################################################
 * resetLeitnerForm
 * ############################################################################
 */
Template.resetLeitnerForm.events({
	"click #resetLeitnerConfirm": function () {
		$('#resetLeitnerModal').on('hidden.bs.modal', function () {
			Meteor.call("deleteLeitner", Router.current().params._id);
		}).modal('hide');
	}
});

/*
 * ############################################################################
 * resetMemoForm
 * ############################################################################
 */
Template.resetMemoForm.events({
	"click #resetMemoConfirm": function () {
		$('#resetMemoModal').on('hidden.bs.modal', function () {
			Meteor.call("deleteWozniak", Router.current().params._id);
		}).modal('hide');
	}
});

/*
* ############################################################################
* learningPhaseInfoBox
* ############################################################################
*/
Template.learningPhaseInfoBox.helpers({
	getDateEnd: function () {
		return moment(this.learningEnd).format("DD.MM.YYYY");
	},
	getDateStart: function () {
		return moment(this.learningStart).format("DD.MM.YYYY");
	},
	getRegistrationPeriod: function () {
		return moment(this.registrationPeriod).format("DD.MM.YYYY");
	},
	getDeadline: function () {
		if (this.daysBeforeReset === 1) {
			return this.daysBeforeReset + " " + TAPi18n.__('panel-body-experience.day');
		} else {
			return this.daysBeforeReset + " " + TAPi18n.__('panel-body-experience.day_plural');
		}
	},
	getLearningStatus: function () {
		if (this.learningEnd.getTime() > new Date().getTime()) {
			return TAPi18n.__('set-list.activeLearnphase');
		} else {
			return TAPi18n.__('set-list.inactiveLearnphase');
		}
	},
	getWorkload: function () {
		if (this.maxCards === 1) {
			return this.maxCards + " " + TAPi18n.__('confirmLearn-form.card');
		} else {
			return this.maxCards + " " + TAPi18n.__('confirmLearn-form.cards');
		}
	},
	canJoinBonus: function () {
		return Bonus.canJoinBonus(Session.get('activeCardset')._id);
	},
	isInBonus: function () {
		return Bonus.isInBonus(Session.get('activeCardset')._id);
	}
});

Template.learningPhaseInfoBox.events({
	"click #collapseLearningPhaseInfoButton": function () {
		changeCollapseIcon("#collapseLearningPhaseInfoIcon");
	}
});

/*
 * ############################################################################
 * leitnerLearning
 * ############################################################################
 */

Template.leitnerLearning.helpers({
	notEmpty: function () {
		return Leitner.find({
			cardset_id: this._id,
			user_id: Meteor.userId(),
			active: true
		}).count();
	}
});

Template.leitnerLearning.events({
	"click #learnBoxActive": function () {
		Router.go('box', {
			_id: Router.current().params._id
		});
	}
});

Template.leitnerLearning.onCreated(function () {
	if (!Roles.userIsInRole(Meteor.userId(), ["admin", "editor"])) {
		if (Session.get('activeCardset').owner !== Meteor.userId() && !Session.get('activeCardset').editors.includes(Meteor.userId())) {
			addToLeitner(Session.get('activeCardset')._id);
		}
	}
});

Template.leitnerLearning.onRendered(function () {
	if (!Roles.userIsInRole(Meteor.userId(), ["admin", "editor"])) {
		if (Meteor.userId() !== Session.get('activeCardset').owner && !Session.get('activeCardset').editors.includes(Meteor.userId())) {
			setTimeout(function () {
				Bert.defaults.hideDelay = 97200;
				let bertType = "success";
				if (Session.get('activeCardset').registrationPeriod.getTime() < new Date()) {
					if (Leitner.findOne({cardset_id: Session.get('activeCardset')._id, user_id: Meteor.userId()}, {fields: {_id: 1}}) === undefined) {
						bertType = "warning";
						BertAlertVisuals.displayBertAlert(TAPi18n.__('bonus.message.registrationPeriodExpired'), bertType, 'growl-top-left');
						return;
					}
				}
				if (Session.get('activeCardset').learningEnd.getTime() > new Date().getTime()) {
					let text = "";
					if (Leitner.find({
						cardset_id: Session.get('activeCardset')._id,
						user_id: Meteor.userId(),
						active: true
					}).count()) {
						var active = Leitner.findOne({
							cardset_id: Session.get('activeCardset')._id,
							user_id: Meteor.userId(),
							active: true
						});
						var deadline = new Date(active.currentDate.getTime() + Session.get('activeCardset').daysBeforeReset * 86400000);
						if (deadline.getTime() > Session.get('activeCardset').learningEnd.getTime()) {
							text += (TAPi18n.__('deadlinePrologue') + moment(Session.get('activeCardset').learningEnd).format("DD.MM.YYYY") + TAPi18n.__('deadlineEpilogue1'));
						} else {
							text += (TAPi18n.__('deadlinePrologue') + moment(deadline).format("DD.MM.YYYY") + TAPi18n.__('deadlineEpilogue2'));
						}
						bertType = "warning";
					} else {
						if (Leitner.find({
							cardset_id: Session.get('activeCardset')._id,
							user_id: Meteor.userId(),
							box: {$ne: 6}
						}).count() === 0) {
							text += TAPi18n.__('bonus.message.learnedEverything');
						} else {
							let nextCardDate = Leitner.findOne({
								cardset_id: Router.current().params._id,
								user_id: Meteor.userId(),
								box: {$ne: 6}
							}, {sort: {nextDate: 1}}).nextDate;
							let learningEnd = Cardsets.findOne({_id: Router.current().params._id}).learningEnd;
							if (nextCardDate.getTime() > learningEnd.getTime()) {
								text += TAPi18n.__('noMoreCardsBeforeEnd');
							}
							let nextDate;
							if (nextCardDate.getTime() < new Date().getTime()) {
								nextDate = moment(new Date());
							} else {
								nextDate = moment(nextCardDate);
							}
							if (nextDate.get('hour') >= Meteor.settings.public.leitner.dayIntervalHour) {
								nextDate.add(1, 'day');
							}
							nextDate.hour(Meteor.settings.public.leitner.dayIntervalHour);
							nextDate.minute(0);
							text += TAPi18n.__('noCardsToLearn') + nextDate.format("DD.MM.YYYY") + TAPi18n.__('at') + nextDate.format("HH:mm") + TAPi18n.__('released');
						}
					}
					BertAlertVisuals.displayBertAlert(text, bertType, 'growl-top-left');
				} else {
					BertAlertVisuals.displayBertAlert(TAPi18n.__('bonus.message.bonusEnded'), bertType, 'growl-top-left');
				}
				Bert.defaults.hideDelay = 7;
			}, 2000);
		}
	}
});
